'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _objectStyles = require('../styles/objectStyles');

var _objectStyles2 = _interopRequireDefault(_objectStyles);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A short description of the object values.
 * Can be used to render tree node in ObjectInspector
 * or render objects in TableInspector.
 */
var ObjectDescription = function ObjectDescription(_ref) {
  var object = _ref.object;

  switch (typeof object === 'undefined' ? 'undefined' : (0, _typeof3.default)(object)) {
    case 'number':
      return _react2.default.createElement(
        'span',
        { style: _objectStyles2.default.value.number },
        object
      );
    case 'string':
      return _react2.default.createElement(
        'span',
        { style: _objectStyles2.default.value.string },
        '"',
        object,
        '"'
      );
    case 'boolean':
      return _react2.default.createElement(
        'span',
        { style: _objectStyles2.default.value.boolean },
        String(object)
      );
    case 'undefined':
      return _react2.default.createElement(
        'span',
        { style: _objectStyles2.default.value.undefined },
        'undefined'
      );
    case 'object':
      if (object === null) {
        return _react2.default.createElement(
          'span',
          { style: _objectStyles2.default.value.null },
          'null'
        );
      }
      if (object instanceof Date) {
        return _react2.default.createElement(
          'span',
          null,
          object.toString()
        );
      }
      if (object instanceof RegExp) {
        return _react2.default.createElement(
          'span',
          { style: _objectStyles2.default.value.regexp },
          object.toString()
        );
      }
      if (Array.isArray(object)) {
        return _react2.default.createElement(
          'span',
          null,
          'Array[' + object.length + ']'
        );
      }
      return _react2.default.createElement(
        'span',
        null,
        object.constructor.name
      );
    case 'function':
      return _react2.default.createElement(
        'span',
        null,
        _react2.default.createElement(
          'span',
          { style: _objectStyles2.default.value.function.keyword },
          'function'
        ),
        _react2.default.createElement(
          'span',
          { style: _objectStyles2.default.value.function.name },
          ' ',
          object.name,
          '()'
        )
      );
    case 'symbol':
      return _react2.default.createElement(
        'span',
        { style: _objectStyles2.default.value.symbol },
        object.toString()
      );
    default:
      return _react2.default.createElement('span', null);
  }
};

ObjectDescription.propTypes = {
  /** the object to describe */
  object: _react2.default.PropTypes.any
};

exports.default = ObjectDescription;